/**************************************************************************
 * InstantLaunch V0.1 - extended instant launch option interpreter
 *     created: Mar., 18th 2006
 * last update: May , 13th 2006
 * Copyright (c) 2006 Friedrich Kastner-Masilko, M.Sc.
 *
 * This module demonstrates how to build an Orbiter plugin that
 * uses the oapiCmdLine() API call and starts a scenario from within
 * orbiter. This is a good starting point for your own in-game scenario
 * creator addons like e.g. star system switchers, economical simulators...
 **************************************************************************/

#define STRICT
#define ORBITER_MODULE
#define _WIN32_WINNT		0x0400

#define LINESIZE			4096				//Default line size
#define DEFAULT_SCNFILE		"(Current state)"	//Default scenario name
#define SCN_FORMAT			"Scenarios\\%s.scn"	//Default scenario path
#define IDC_STARTORBI		0x03E8				//Orbiter's Start button
#define IDC_EXITORBI		0x03E9				//Orbiter's Exit button
#define IDC_SELECTTREE		0x0440				//Orbiter's Scenario selector
#define IDC_PAUSED			0x0442				//Orbiter's Pausing checkbox
#define TABPAGESKIP			5					//How many windows next from Start buttom to tab page
#define ERROR_APPLICATION	0x20000000			//Bit 29 for application errors

#include "windows.h"
#include "orbitersdk.h"
#include "resource.h"
#include <stdio.h>
#include <commctrl.h>

//Global variables
HINSTANCE g_hInst;
HWND hMain;

//Structure storing window handles and some flags
struct windowinfo
{
	//Orbiter elements
	HWND		idcstartorbi,
				idcexitorbi,
				idcselecttree,
				idcpaused,
				idcorbiter;
	BOOL		exitflag,
				exitonerror,
				showhelp;
}
winfo;

//Prototypes
BOOL CALLBACK MsgProc (HWND, UINT, WPARAM, LPARAM);
BOOL CALLBACK MsgProc2 (HWND, UINT, WPARAM, LPARAM);
BOOL CALLBACK MsgProc3 (HWND, UINT, WPARAM, LPARAM);

//Structure for command line parser
struct parsblock
{
			char		*block;
	struct parsblock	*next;
};

//Parse step routine
void parsstep(int i, int &j, int &k, char *line, struct parsblock **current)
{
	line[i]=0x00;	
	int z;
	if (j<i)
	{
		current[0]->block=new char[z=i-j+1];		
		strncpy(current[0]->block, line+j, z);
		current[0]=current[0]->next=new parsblock;
		k++;
		j=i;
	}
	j++;
}

//Command line parser
int pars(char *line, char ***args, int argc)
{
	struct parsblock *blocks, *current;
	int len=strlen(line), i=0, j=0, k=0;
	bool quoted=false;	

	//Clear old field
	if (args[0]!=0 && argc>0)
	{
		for(;argc>0;) delete [] args[0][--argc];
		delete [] args[0];
	}

	//Create root
	blocks=current=new parsblock;
	//Create parsblocks
	for(;i<len;i++)
	{
		switch(line[i])
		{
		case '"':
			if (quoted)
			{
				//Quote end
				parsstep(i, j, k, line, &current);
				quoted=false;				
			}
			else
			{
				//Quote start
				parsstep(i, j, k, line, &current);
				quoted=true;
			}
			break;
		case ' ':
		case '\t':
			if (!quoted)
			{
				//Whitespace
				parsstep(i, j, k, line, &current);
			}
			break;
		}
	}
	parsstep(i, j, k, line, &current);
	//Convert parsblocks
	if (k>0) args[0]= new char*[k];
	else args[0]=0;
	for(i=0; i<k; i++)
	{
		args[0][i]=blocks->block;
		current=blocks;
		blocks=blocks->next;
		delete current;
	}
	delete blocks;
	return k;
}

//Callback for EnumWindows(2) - Searches for the Orbiter launchpad window
BOOL CALLBACK ScanForOrbiter(HWND hWnd, LPARAM lparam)
{
	DWORD hinstance;	
	long *conf_long=(long *)lparam;
	GetWindowThreadProcessId(hWnd, &hinstance);
	if (((HINSTANCE)hinstance)==((HINSTANCE)conf_long[0]))
		if (IsWindowVisible(hWnd))
		{
			SetLastError(ERROR_APPLICATION);
			conf_long[0]=(long)hWnd;
			return FALSE;
		}	
	return TRUE;
}

DLLCLBK void InitModule (HINSTANCE hDLL)
{
	g_hInst = hDLL; // remember the instance handle
	long conf_long, conf_long2;
	int i, j, k, argc=0;
	char **args=0;

	winfo.exitonerror=false;
	winfo.showhelp=true;

	//Find Orbiter's main windows
	conf_long=(long)GetCurrentProcessId();
	if (EnumWindows(ScanForOrbiter, (LPARAM)&conf_long)) exit (-1);
	if ((conf_long2=GetLastError())!=ERROR_APPLICATION)	exit (-1);

	//Get Orbiter's Start button
	winfo.idcorbiter=(HWND)conf_long;
	winfo.idcstartorbi=GetDlgItem(winfo.idcorbiter, IDC_STARTORBI);
	if (winfo.idcstartorbi==NULL) exit (-1);	

	//Get Orbiter's Exit button
	winfo.idcorbiter=(HWND)conf_long;
	winfo.idcexitorbi=GetDlgItem(winfo.idcorbiter, IDC_EXITORBI);
	if (winfo.idcexitorbi==NULL) exit (-1);
	winfo.exitflag=false;

	//Get the first tab page's window handle
	conf_long=(long)winfo.idcstartorbi;
	for(i=0;i<TABPAGESKIP;i++) conf_long=(long)GetNextWindow((HWND)conf_long, GW_HWNDNEXT);
	if (conf_long==NULL) exit (-1);

	//Get Orbiter's selection tree
	winfo.idcselecttree=GetDlgItem((HWND)conf_long, IDC_SELECTTREE);
	if (winfo.idcselecttree==NULL) exit (-1);

	if (!TreeView_SelectItem(winfo.idcselecttree, TreeView_GetRoot(winfo.idcselecttree))) exit (-1);
	
	//Get Orbiter's Paused checkbox
	winfo.idcpaused=GetDlgItem((HWND)conf_long, IDC_PAUSED);
	if (winfo.idcpaused==NULL) exit (-1);

	SendMessage(winfo.idcpaused, BM_SETCHECK, BST_UNCHECKED, 0);
	
	TCHAR source[LINESIZE]="", target[LINESIZE]="";

	strcpy(source, oapiGetCmdLine());
	argc=pars(source, &args, argc);
	if (argc>0)
	{
		for(i=0;i<argc;i++)
			if (args[i][0]=='-')
			{
				k=strlen(args[i]);
				for (j=1;j<k;j++)
				{
					switch (args[i][j])
					{
					case 'p':
					case 'P':
						SendMessage(winfo.idcpaused, BM_SETCHECK, BST_CHECKED, 0);
						break;
					case 'e':
					case 'E':
						winfo.exitflag=true;
						break;
					case 'x':
					case 'X':
						winfo.exitonerror=true;
						break;
					}
				}
			}
			else strcpy(target, args[i]);		
	}
	else winfo.showhelp=false;
		
	if (GetLongPathName(target, source, LINESIZE)==0) strcpy(source, target);	
	for(i=strlen(source)-1;i>=0 && source[i]!='.';i--)
		if (source[i]=='\\' || source[i]=='/')
		{
			strcat(source, ".scn");
			break;
		}
	sprintf(target, SCN_FORMAT, DEFAULT_SCNFILE);

	if (CopyFile(source, target, false)) hMain = oapiOpenDialog(g_hInst, IDD_DEBUG, MsgProc);
	else if (winfo.exitonerror) hMain = oapiOpenDialog(g_hInst, IDD_DEBUG, MsgProc2);
	else if (winfo.showhelp) hMain = oapiOpenDialog(g_hInst, IDD_USAGE, MsgProc3);
}

DLLCLBK void ExitModule (HINSTANCE hDLL)
{	
	if (hMain!=NULL) oapiCloseDialog (hMain);	
	hMain=NULL;
}

DLLCLBK void opcOpenRenderViewport (HWND hWnd, DWORD w, DWORD h, BOOL fullscreen)
{
	if (hMain!=NULL) oapiCloseDialog (hMain);
	hMain=NULL;
}

DLLCLBK void opcCloseRenderViewport()
{
	if (winfo.exitflag) hMain = oapiOpenDialog(g_hInst, IDD_DEBUG, MsgProc2);
}

//MsgProc for the starting window
BOOL CALLBACK MsgProc (HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{	
	switch (uMsg) {
	case WM_SHOWWINDOW:		
		PostMessage(winfo.idcstartorbi, BM_CLICK, 0, 0);
		return TRUE;
	}
	return oapiDefDialogProc (hDlg, uMsg, wParam, lParam);
}

//MsgProc for the closing window
BOOL CALLBACK MsgProc2 (HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{	
	if (IsWindowVisible(winfo.idcorbiter))
		PostMessage(winfo.idcexitorbi, BM_CLICK, 0, 0);
	return FALSE;	
}

//MsgProc for the usage window
BOOL CALLBACK MsgProc3 (HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{	
	switch (uMsg) {
	case WM_CLOSE:		
		if (hMain!=NULL) oapiCloseDialog (hMain);
		hMain=NULL;
		return TRUE;
	}
	return oapiDefDialogProc (hDlg, uMsg, wParam, lParam);	
}